
// Murobo Control Center Hardware Input Program
// Version 1.0
// MCU: ATMEGA2560
// Arduino IDE: 2.1.1

#define STATE_IDLE            0
#define STATE_EMERGENCYSTOP   1
#define STATE_CYCLESTOP       2
#define STATE_CYCLESTART      3
#define STATE_FEEDHOLD        4
#define STATE_WHEELPOS        5
#define STATE_WHEELAXIS       6
#define STATE_WHEELFEED       7
#define STATE_WHEELMODE       8
#define STATE_FEEDOVERRIDE    9
#define STATE_SPINDLEOVERRIDE 10
#define STATE_MACRO           11

#define STATE_END          STATE_MACRO + 1

#define REPORT_START        "["
#define REPORT_DELIMITER    ","
#define REPORT_END          "]"
#define REPORT_REQUEST      '?'

#define JOGFEED_SLOW         "1"
#define JOGFEED_MEDIUM       "3"
#define JOGFEED_FAST         "6"

#define WHEELMODE_AUTO       "0"
#define WHEELMODE_DISTANCE   "1"
#define WHEELMODE_VELOCITY   "2"

#define READ_PIN_REGISTERS 1

// The position of the hand wheel
int CurrentWheelPos = 0;

char PrevWheelA = -1;
char PrevWheelB = -1;
char PrevWheelAN = -1;
char PrevWheelBN = -1;

unsigned long displayTime = 0;
unsigned long numCalls = 0;
unsigned long numSerialCalls = 0;


// State values
char AxisX;
char AxisY;
char AxisZ;
char Axis4;
char EmergencyStop;
char FeedFast;
char FeedMedium;
char FeedSlow;
int WheelPos;

void setup() {
  PrevWheelA = -1;
  PrevWheelB = -1;
  PrevWheelAN = -1;
  PrevWheelBN = -1;

  pinMode(22, INPUT_PULLUP);
  pinMode(23, INPUT_PULLUP);
  pinMode(24, INPUT);
  pinMode(24, INPUT);
  pinMode(25, INPUT);
  pinMode(26, INPUT);
  pinMode(27, INPUT);
  pinMode(28, INPUT_PULLUP);
  pinMode(29, INPUT_PULLUP);
  pinMode(30, INPUT_PULLUP);
  pinMode(31, INPUT_PULLUP);
  pinMode(32, INPUT_PULLUP);
  pinMode(33, INPUT_PULLUP);

  Serial.begin(115200);
  if (Serial) {
    Serial.print("MCC 0.3\n");
  }  
}

int CurrentState = STATE_IDLE;

void processHandWheel() {
#if READ_PIN_REGISTERS
  char wheelB = (PINA & (1<<PA4)) ? '1' : '0'; // Pin 26
  char wheelA = (PINA & (1<<PA5)) ? '1' : '0'; // Pin 27
  char wheelBN = (PINA & (1<<PA2)) ? '1' : '0'; // Pin 24
  char wheelAN = (PINA & (1<<PA3)) ? '1' : '0'; // Pin 25
#else
  char wheelB = digitalRead(26) ? '1' : '0';
  char wheelA = digitalRead(27) ? '1' : '0';
  char wheelBN = digitalRead(24) ? '1' : '0';
  char wheelAN = digitalRead(25) ? '1' : '0';  
#endif

  if (wheelA != PrevWheelA || wheelB != PrevWheelB) {
    if (wheelA == '1' && wheelB == '1' && wheelAN == '0' && wheelBN == '0') {
      if (PrevWheelA == '1' && PrevWheelB != '1' && PrevWheelAN != '1' && PrevWheelBN == '1') {
        CurrentWheelPos++;
      } else if (PrevWheelA != '1' && PrevWheelB == '1' && PrevWheelAN == '1' && PrevWheelBN != '1') {
        CurrentWheelPos--;
      } else {
        //Serial.print("HandWheel Error: ");
        //Serial.print(PrevWheelA);
        //Serial.println(PrevWheelB);
      }
    }
    PrevWheelA = wheelA;
    PrevWheelB = wheelB;
    PrevWheelAN = wheelAN;
    PrevWheelBN = wheelBN;
  } // end if wheel moved  
}

void loop() {

  // Process the input from the handwheel
  processHandWheel();
    
  // Store the current pin and wheel values and prepare to output to the serial
  // buffer in pieces so as not to slow down transmission
  if (CurrentState == STATE_IDLE) {    
#if READ_PIN_REGISTERS
      AxisX = (PINA & (1<<PA1)) ? '1' : '0'; // Pin 23 
      AxisY = (PINA & (1<<PA0)) ? '1' : '0'; // Pin 22
      AxisZ = (PINC & (1<<PC4)) ? '1' : '0'; // Pin 33
      Axis4 = (PINC & (1<<PC5)) ? '1' : '0'; // Pin 32
      EmergencyStop = (PINA & (1<<PA6)) ? '1' : '0'; // Pin 28
      FeedFast =      (PINA & (1<<PA7)) ? '1' : '0'; // Pin 29
      FeedMedium =    (PINC & (1<<PC7)) ? '1' : '0'; // Pin 30
      FeedSlow =      (PINC & (1<<PC6)) ? '1' : '0'; // Pin 31
#else
      AxisX = digitalRead(23) ? '1' : '0';
      AxisY = digitalRead(22) ? '1' : '0';
      AxisZ = digitalRead(33) ? '1' : '0';
      Axis4 = digitalRead(32) ? '1' : '0';    
      EmergencyStop = digitalRead(28) ? '1' : '0';
      FeedFast = digitalRead(29) ? '1' : '0';
      FeedMedium = digitalRead(30) ? '1' : '0';
      FeedSlow = digitalRead(31) ? '1' : '0';
#endif      
      WheelPos = CurrentWheelPos;      
      
      // Fullfill the request to output data if there was one
      if (Serial.available()) {
        int cmd = Serial.read();
        if (cmd == REPORT_REQUEST) {
          Serial.print(REPORT_START);
          numSerialCalls++;
          CurrentState++;
        } 
      }
  } else if (CurrentState == STATE_EMERGENCYSTOP) {      
      Serial.print(EmergencyStop == '1' ? "1" : "0");
      CurrentState++;
  } else if (CurrentState == STATE_CYCLESTOP) {
      // Handwheel doesn't support cycle stop button
      Serial.print(REPORT_DELIMITER);
      Serial.print("0");
      CurrentState++;
  } else if (CurrentState == STATE_CYCLESTART) {
      // Handwheel doesn't support cycle start button
      Serial.print(REPORT_DELIMITER);
      Serial.print("0");
      CurrentState++;
  }  else if (CurrentState == STATE_FEEDHOLD) {
      // Handwheel doesn't support feed hold button
      Serial.print(REPORT_DELIMITER);
      Serial.print("0");
      CurrentState++;
  } else if (CurrentState == STATE_WHEELPOS) {     
      Serial.print(REPORT_DELIMITER); 
      Serial.print(WheelPos, DEC);      
      CurrentState++;
  } else if (CurrentState == STATE_WHEELAXIS ) {
      if (AxisX == '0') {        
        Serial.print(REPORT_DELIMITER);
        Serial.print("1");
      } else if (AxisY == '0') {        
        Serial.print(REPORT_DELIMITER);
        Serial.print("2");        
      } else if (AxisZ == '0') { 
        Serial.print(REPORT_DELIMITER);
        Serial.print("3");
      } else if (Axis4 == '0') {
        Serial.print(REPORT_DELIMITER);        
        Serial.print("4");
      } else {
        Serial.print(REPORT_DELIMITER);
        Serial.print("0");
      }
      CurrentState++;
  } else if (CurrentState == STATE_WHEELFEED) {
      if (FeedSlow == '0') {       
        Serial.print(REPORT_DELIMITER);
        Serial.print(JOGFEED_SLOW);
      } else if (FeedMedium == '0') {       
        Serial.print(REPORT_DELIMITER); 
        Serial.print(JOGFEED_MEDIUM);
      } else if (FeedFast == '0') {
        Serial.print(REPORT_DELIMITER);
        Serial.print(JOGFEED_FAST);
      } else {        
        Serial.print(REPORT_DELIMITER);
        Serial.print("0");
      }
      CurrentState++;
  } else if (CurrentState == STATE_WHEELMODE) {
       Serial.print(REPORT_DELIMITER);
       Serial.print(WHEELMODE_AUTO);
       CurrentState++;
  } else if (CurrentState == STATE_FEEDOVERRIDE) {
      // This handwheel doesn't have a feed override knob
      Serial.print(REPORT_DELIMITER);
      Serial.print(" ");
      CurrentState++;
  } else if (CurrentState == STATE_SPINDLEOVERRIDE) {
      // This handwheel doesn't have a speed override knob      
      Serial.print(REPORT_DELIMITER);
      Serial.print(" ");
      CurrentState++;
  } else if (CurrentState == STATE_MACRO) {
      // Handwheel doesn't support macro buttons
      Serial.print(REPORT_DELIMITER);
      Serial.print("0");
      CurrentState++;
  }

  // Restart the loop
  if (CurrentState == STATE_END) {
    Serial.print(REPORT_END);
    Serial.print("\n");
    CurrentState = STATE_IDLE;
  }

  // Show the call frequency only when in the idle state so we don't interrupt a status report
#if 0
  numCalls++;

  if (CurrentState == STATE_IDLE) {
    unsigned long currentTime = millis();
    if (displayTime == 0) {
      displayTime = currentTime;
    }
    
    if (currentTime - displayTime > 1000) {
      Serial.print(1.0f * numCalls / (currentTime - displayTime));
      Serial.print(" ");
      Serial.print(1.0f * numSerialCalls / (currentTime - displayTime));
      Serial.print(" KHz Call Rate\n");
      displayTime = millis();;
      numCalls = 0;
      numSerialCalls = 0;
    }
  }
#endif

}


